/*
 * Copyright (c) 1989 The Regents of the University of California.
 * All rights reserved.
 * (c) UNIX System Laboratories, Inc.
 * All or some portions of this file are derived from material licensed
 * to the University of California by American Telephone and Telegraph
 * Co. or Unix System Laboratories, Inc. and are reproduced herein with
 * the permission of UNIX System Laboratories, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include "ctype.h"

#include <stdio.h>
#include <ctype.h>

#define CTYPE_NUM_CHARS       256

#define _CTYPE_U 0x01
#define _CTYPE_L 0x02
#define _CTYPE_D 0x04
#define _CTYPE_S 0x08
#define _CTYPE_P 0x10
#define _CTYPE_C 0x20
#define _CTYPE_X 0x40
#define _CTYPE_B 0x80
#define _CTYPE_R (_CTYPE_P|_CTYPE_U|_CTYPE_L|_CTYPE_D|_CTYPE_B)
#define _CTYPE_A (_CTYPE_L|_CTYPE_U)
#define _CTYPE_N _CTYPE_D

const short _hybris_C_tolower_[1 + CTYPE_NUM_CHARS] = {
        EOF,
        0x00,	0x01,	0x02,	0x03,	0x04,	0x05,	0x06,	0x07,
        0x08,	0x09,	0x0a,	0x0b,	0x0c,	0x0d,	0x0e,	0x0f,
        0x10,	0x11,	0x12,	0x13,	0x14,	0x15,	0x16,	0x17,
        0x18,	0x19,	0x1a,	0x1b,	0x1c,	0x1d,	0x1e,	0x1f,
        0x20,	0x21,	0x22,	0x23,	0x24,	0x25,	0x26,	0x27,
        0x28,	0x29,	0x2a,	0x2b,	0x2c,	0x2d,	0x2e,	0x2f,
        0x30,	0x31,	0x32,	0x33,	0x34,	0x35,	0x36,	0x37,
        0x38,	0x39,	0x3a,	0x3b,	0x3c,	0x3d,	0x3e,	0x3f,
        0x40,	'a',	'b',	'c',	'd',	'e',	'f',	'g',
        'h',	'i',	'j',	'k',	'l',	'm',	'n',	'o',
        'p',	'q',	'r',	's',	't',	'u',	'v',	'w',
        'x',	'y',	'z',	0x5b,	0x5c,	0x5d,	0x5e,	0x5f,
        0x60,	0x61,	0x62,	0x63,	0x64,	0x65,	0x66,	0x67,
        0x68,	0x69,	0x6a,	0x6b,	0x6c,	0x6d,	0x6e,	0x6f,
        0x70,	0x71,	0x72,	0x73,	0x74,	0x75,	0x76,	0x77,
        0x78,	0x79,	0x7a,	0x7b,	0x7c,	0x7d,	0x7e,	0x7f,
        0x80,	0x81,	0x82,	0x83,	0x84,	0x85,	0x86,	0x87,
        0x88,	0x89,	0x8a,	0x8b,	0x8c,	0x8d,	0x8e,	0x8f,
        0x90,	0x91,	0x92,	0x93,	0x94,	0x95,	0x96,	0x97,
        0x98,	0x99,	0x9a,	0x9b,	0x9c,	0x9d,	0x9e,	0x9f,
        0xa0,	0xa1,	0xa2,	0xa3,	0xa4,	0xa5,	0xa6,	0xa7,
        0xa8,	0xa9,	0xaa,	0xab,	0xac,	0xad,	0xae,	0xaf,
        0xb0,	0xb1,	0xb2,	0xb3,	0xb4,	0xb5,	0xb6,	0xb7,
        0xb8,	0xb9,	0xba,	0xbb,	0xbc,	0xbd,	0xbe,	0xbf,
        0xc0,	0xc1,	0xc2,	0xc3,	0xc4,	0xc5,	0xc6,	0xc7,
        0xc8,	0xc9,	0xca,	0xcb,	0xcc,	0xcd,	0xce,	0xcf,
        0xd0,	0xd1,	0xd2,	0xd3,	0xd4,	0xd5,	0xd6,	0xd7,
        0xd8,	0xd9,	0xda,	0xdb,	0xdc,	0xdd,	0xde,	0xdf,
        0xe0,	0xe1,	0xe2,	0xe3,	0xe4,	0xe5,	0xe6,	0xe7,
        0xe8,	0xe9,	0xea,	0xeb,	0xec,	0xed,	0xee,	0xef,
        0xf0,	0xf1,	0xf2,	0xf3,	0xf4,	0xf5,	0xf6,	0xf7,
        0xf8,	0xf9,	0xfa,	0xfb,	0xfc,	0xfd,	0xfe,	0xff
};

const short _hybris_C_toupper_[1 + CTYPE_NUM_CHARS] = {
        EOF,
        0x00,	0x01,	0x02,	0x03,	0x04,	0x05,	0x06,	0x07,
        0x08,	0x09,	0x0a,	0x0b,	0x0c,	0x0d,	0x0e,	0x0f,
        0x10,	0x11,	0x12,	0x13,	0x14,	0x15,	0x16,	0x17,
        0x18,	0x19,	0x1a,	0x1b,	0x1c,	0x1d,	0x1e,	0x1f,
        0x20,	0x21,	0x22,	0x23,	0x24,	0x25,	0x26,	0x27,
        0x28,	0x29,	0x2a,	0x2b,	0x2c,	0x2d,	0x2e,	0x2f,
        0x30,	0x31,	0x32,	0x33,	0x34,	0x35,	0x36,	0x37,
        0x38,	0x39,	0x3a,	0x3b,	0x3c,	0x3d,	0x3e,	0x3f,
        0x40,	0x41,	0x42,	0x43,	0x44,	0x45,	0x46,	0x47,
        0x48,	0x49,	0x4a,	0x4b,	0x4c,	0x4d,	0x4e,	0x4f,
        0x50,	0x51,	0x52,	0x53,	0x54,	0x55,	0x56,	0x57,
        0x58,	0x59,	0x5a,	0x5b,	0x5c,	0x5d,	0x5e,	0x5f,
        0x60,	'A',	'B',	'C',	'D',	'E',	'F',	'G',
        'H',	'I',	'J',	'K',	'L',	'M',	'N',	'O',
        'P',	'Q',	'R',	'S',	'T',	'U',	'V',	'W',
        'X',	'Y',	'Z',	0x7b,	0x7c,	0x7d,	0x7e,	0x7f,
        0x80,	0x81,	0x82,	0x83,	0x84,	0x85,	0x86,	0x87,
        0x88,	0x89,	0x8a,	0x8b,	0x8c,	0x8d,	0x8e,	0x8f,
        0x90,	0x91,	0x92,	0x93,	0x94,	0x95,	0x96,	0x97,
        0x98,	0x99,	0x9a,	0x9b,	0x9c,	0x9d,	0x9e,	0x9f,
        0xa0,	0xa1,	0xa2,	0xa3,	0xa4,	0xa5,	0xa6,	0xa7,
        0xa8,	0xa9,	0xaa,	0xab,	0xac,	0xad,	0xae,	0xaf,
        0xb0,	0xb1,	0xb2,	0xb3,	0xb4,	0xb5,	0xb6,	0xb7,
        0xb8,	0xb9,	0xba,	0xbb,	0xbc,	0xbd,	0xbe,	0xbf,
        0xc0,	0xc1,	0xc2,	0xc3,	0xc4,	0xc5,	0xc6,	0xc7,
        0xc8,	0xc9,	0xca,	0xcb,	0xcc,	0xcd,	0xce,	0xcf,
        0xd0,	0xd1,	0xd2,	0xd3,	0xd4,	0xd5,	0xd6,	0xd7,
        0xd8,	0xd9,	0xda,	0xdb,	0xdc,	0xdd,	0xde,	0xdf,
        0xe0,	0xe1,	0xe2,	0xe3,	0xe4,	0xe5,	0xe6,	0xe7,
        0xe8,	0xe9,	0xea,	0xeb,	0xec,	0xed,	0xee,	0xef,
        0xf0,	0xf1,	0xf2,	0xf3,	0xf4,	0xf5,	0xf6,	0xf7,
        0xf8,	0xf9,	0xfa,	0xfb,	0xfc,	0xfd,	0xfe,	0xff
};


#define _U _CTYPE_U
#define _L _CTYPE_L
#define _N _CTYPE_N
#define _S _CTYPE_S
#define _P _CTYPE_P
#define _C _CTYPE_C
#define _X _CTYPE_X
#define _B _CTYPE_B
const char _hybris_C_ctype_[1 + CTYPE_NUM_CHARS] = {
        0,
        _C,	_C,	_C,	_C,	_C,	_C,	_C,	_C,
        _C,	_C|_S,	_C|_S,	_C|_S,	_C|_S,	_C|_S,	_C,	_C,
        _C,	_C,	_C,	_C,	_C,	_C,	_C,	_C,
        _C,	_C,	_C,	_C,	_C,	_C,	_C,	_C,
        _S|(char)_B,	_P,	_P,	_P,	_P,	_P,	_P,	_P,
        _P,	_P,	_P,	_P,	_P,	_P,	_P,	_P,
        _N,	_N,	_N,	_N,	_N,	_N,	_N,	_N,
        _N,	_N,	_P,	_P,	_P,	_P,	_P,	_P,
        _P,	_U|_X,	_U|_X,	_U|_X,	_U|_X,	_U|_X,	_U|_X,	_U,
        _U,	_U,	_U,	_U,	_U,	_U,	_U,	_U,
        _U,	_U,	_U,	_U,	_U,	_U,	_U,	_U,
        _U,	_U,	_U,	_P,	_P,	_P,	_P,	_P,
        _P,	_L|_X,	_L|_X,	_L|_X,	_L|_X,	_L|_X,	_L|_X,	_L,
        _L,	_L,	_L,	_L,	_L,	_L,	_L,	_L,
        _L,	_L,	_L,	_L,	_L,	_L,	_L,	_L,
        _L,	_L,	_L,	_P,	_P,	_P,	_P,	_C,

        0,	 0,	 0,	 0,	 0,	 0,	 0,	 0, /* 80 */
        0,	 0,	 0,	 0,	 0,	 0,	 0,	 0, /* 88 */
        0,	 0,	 0,	 0,	 0,	 0,	 0,	 0, /* 90 */
        0,	 0,	 0,	 0,	 0,	 0,	 0,	 0, /* 98 */
        0,	 0,	 0,	 0,	 0,	 0,	 0,	 0, /* A0 */
        0,	 0,	 0,	 0,	 0,	 0,	 0,	 0, /* A8 */
        0,	 0,	 0,	 0,	 0,	 0,	 0,	 0, /* B0 */
        0,	 0,	 0,	 0,	 0,	 0,	 0,	 0, /* B8 */
        0,	 0,	 0,	 0,	 0,	 0,	 0,	 0, /* C0 */
        0,	 0,	 0,	 0,	 0,	 0,	 0,	 0, /* C8 */
        0,	 0,	 0,	 0,	 0,	 0,	 0,	 0, /* D0 */
        0,	 0,	 0,	 0,	 0,	 0,	 0,	 0, /* D8 */
        0,	 0,	 0,	 0,	 0,	 0,	 0,	 0, /* E0 */
        0,	 0,	 0,	 0,	 0,	 0,	 0,	 0, /* E8 */
        0,	 0,	 0,	 0,	 0,	 0,	 0,	 0, /* F0 */
        0,	 0,	 0,	 0,	 0,	 0,	 0,	 0  /* F8 */
};


const short *_hybris_tolower_tab_ = _hybris_C_tolower_;
const short *_hybris_toupper_tab_ = _hybris_C_toupper_;
const char *_hybris_ctype_ = _hybris_C_ctype_;


int
hybris_isalnum(int c)
{
    return (c == EOF ? 0 : ((_hybris_ctype_ + 1)[(unsigned char)c] & (_U|_L|_N)));
}

int
hybris_isalpha(int c)
{
    return (c == EOF ? 0 : ((_hybris_ctype_ + 1)[(unsigned char)c] & (_U|_L)));
}

int
hybris_isblank(int c)
{
    return (c == ' ' || c == '\t');
}

int
hybris_iscntrl(int c)
{
    return (c == EOF ? 0 : ((_hybris_ctype_ + 1)[(unsigned char)c] & _C));
}

int
hybris_isdigit(int c)
{
    return (c == EOF ? 0 : ((_hybris_ctype_ + 1)[(unsigned char)c] & _N));
}

int
hybris_isgraph(int c)
{
    return (c == EOF ? 0 : ((_hybris_ctype_ + 1)[(unsigned char)c] & (_P|_U|_L|_N)));
}

int
hybris_islower(int c)
{
    return (c == EOF ? 0 : ((_hybris_ctype_ + 1)[(unsigned char)c] & _L));
}

int
hybris_isprint(int c)
{
    return (c == EOF ? 0 : ((_hybris_ctype_ + 1)[(unsigned char)c] & (_P|_U|_L|_N|_B)));
}

int
hybris_ispunct(int c)
{
    return (c == EOF ? 0 : ((_hybris_ctype_ + 1)[(unsigned char)c] & _P));
}

int
hybris_isspace(int c)
{
    return (c == EOF ? 0 : ((_hybris_ctype_ + 1)[(unsigned char)c] & _S));
}

int
hybris_isupper(int c)
{
    return (c == EOF ? 0 : ((_hybris_ctype_ + 1)[(unsigned char)c] & _U));
}

int
hybris_isxdigit(int c)
{
    return (c == EOF ? 0 : ((_hybris_ctype_ + 1)[(unsigned char)c] & (_N|_X)));
}
